@@ -48,7 +48,8 @@ define([
                 'csv': 'csv',
                 'excel': 'xlsx',
                 'json': 'json',
-                'pickle': ''
+                'pickle': '',
+                'parquet': 'parquet'
             }
             this.dataPath = 'https://raw.githubusercontent.com/visualpython/visualpython/main/visualpython/data/sample_csv/';
             this.fileResultState = {
@@ -60,7 +61,8 @@ define([
                     'csv': 'pd_toCsv',
                     'excel': 'pd_toExcel',
                     'json': 'pd_toJson',
-                    'pickle': 'pd_toPickle'
+                    'pickle': 'pd_toPickle',
+                    'parquet': 'pd_toParquet'
                 },
                 selectedType: 'csv',
                 package: null
