@@ -260,6 +260,10 @@ define([
                     that.config.checkModules = ['pd', 'vp_create_feature_importances'];
                 } else if (name == 'plot_feature_importances') {
                     that.config.checkModules = ['pd', 'plt', 'vp_create_feature_importances', 'vp_plot_feature_importances'];
+                } else if (name == 'permutation_importance') {
+                    that.config.checkModules = ['pd', 'vp_create_permutation_importances'];
+                } else if (name == 'plot_permutation_importance') {
+                    that.config.checkModules = ['pd', 'vp_create_permutation_importances', 'vp_plot_permutation_importances'];
                 } else {
                     that.config.checkModules = ['pd'];
                 }
@@ -409,16 +413,46 @@ define([
                     name: 'permutation_importance',
                     label: 'Permutation importance',
                     import: 'from sklearn.inspection import permutation_importance',
-                    code: '${importance_allocate} = permutation_importance(${model}, ${importance_featureData}, ${importance_targetData}${scoring}${random_state}${etc})',
+                    code: '${importance_allocate} = vp_create_permutation_importances(${model}, ${importance_featureData}, ${importance_targetData}${scoring}${sort})',
                     description: 'Permutation importance for feature evaluation.',
                     options: [
                         { name: 'importance_featureData', label: 'Feature Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'X_train' },
                         { name: 'importance_targetData', label: 'Target Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'y_train' },
-                        { name: 'scoring', component: ['input'], usePair: true },
-                        { name: 'random_state', component: ['input_number'], placeholder: '123', usePair: true },
+                        { name: 'scoring', component: ['option_suggest'], usePair: true, type: 'text', 
+                            options: [
+                                'explained_variance', 'max_error', 'neg_mean_absolute_error', 'neg_mean_squared_error', 'neg_root_mean_squared_error',
+                                'neg_mean_squared_log_error', 'neg_median_absolute_error', 'r2', 'neg_mean_poisson_deviance', 'neg_mean_gamma_deviance',
+                                'neg_mean_absolute_percentage_error',
+                                'accuracy', 'balanced_accuracy', 'top_k_accuracy', 'average_precision', 'neg_brier_score',
+                                'f1', 'f1_micro', 'f1_macro', 'f1_weighted', 'f1_samples', 'neg_log_loss', 'precision', 'recall', 'jaccard', 
+                                'roc_auc', 'roc_auc_ovr', 'roc_auc_ovo', 'roc_auc_ovr_weighted', 'roc_auc_ovo_weighted'
+                            ] },
+                        { name: 'sort', label: 'Sort data', component: ['bool_checkbox'], value: true, usePair: true },
                         { name: 'importance_allocate', label: 'Allocate to', component: ['input'], placeholder: 'New variable', value: 'importances' }
                     ]
                 },
+                'plot_permutation_importance': {
+                    name: 'plot_permutation_importance',
+                    label: 'Plot permutation importance',
+                    import: 'from sklearn.inspection import permutation_importance',
+                    code: 'vp_plot_permutation_importances(${model}, ${importance_featureData}, ${importance_targetData}${scoring}${sort}${top_count})',
+                    description: 'Permutation importance for feature evaluation.',
+                    options: [
+                        { name: 'importance_featureData', label: 'Feature Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'X_train' },
+                        { name: 'importance_targetData', label: 'Target Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'y_train' },
+                        { name: 'scoring', component: ['option_suggest'], usePair: true, type: 'text', 
+                            options: [
+                                'explained_variance', 'max_error', 'neg_mean_absolute_error', 'neg_mean_squared_error', 'neg_root_mean_squared_error',
+                                'neg_mean_squared_log_error', 'neg_median_absolute_error', 'r2', 'neg_mean_poisson_deviance', 'neg_mean_gamma_deviance',
+                                'neg_mean_absolute_percentage_error',
+                                'accuracy', 'balanced_accuracy', 'top_k_accuracy', 'average_precision', 'neg_brier_score',
+                                'f1', 'f1_micro', 'f1_macro', 'f1_weighted', 'f1_samples', 'neg_log_loss', 'precision', 'recall', 'jaccard', 
+                                'roc_auc', 'roc_auc_ovr', 'roc_auc_ovo', 'roc_auc_ovr_weighted', 'roc_auc_ovo_weighted'
+                            ] },
+                        { name: 'sort', label: 'Sort data', component: ['bool_checkbox'], value: true, usePair: true },
+                        { name: 'top_count', label: 'Top count', component: ['input_number'], min: 0, usePair: true }
+                    ]
+                },
                 'feature_importances': {
                     name: 'feature_importances',
                     label: 'Feature importances',
@@ -514,6 +548,19 @@ define([
                             }
                         }
                     }
+                    if (modelType === 'SMOTE') {
+                        infos = {
+                            'get_feature_names_out': {
+                                name: 'get_feature_names_out',
+                                label: 'Get feature names',
+                                code: '${feature_names_allocate} = ${model}.get_feature_names_out()',
+                                description: 'Get output feature names for transformation.',
+                                options: [
+                                    { name: 'feature_names_allocate', label: 'Allocate to', component: ['input'], placeholder: 'New variable', value: 'features' }
+                                ]
+                            }
+                        }
+                    }
                     infos = {
                         ...infos,
                         'get_params': defaultInfos['get_params']
@@ -545,7 +592,44 @@ define([
                                 { name: 'cvs_allocate', label: 'Allocate to', component: ['input'], placeholder: 'New variable', value: 'scores' }
                             ]
                         },
-                        'permutation_importance': defaultInfos['permutation_importance'],
+                        'permutation_importance': {
+                            name: 'permutation_importance',
+                            label: 'Permutation importance',
+                            import: 'from sklearn.inspection import permutation_importance',
+                            code: '${importance_allocate} = vp_create_permutation_importances(${model}, ${importance_featureData}, ${importance_targetData}${scoring}${sort})',
+                            description: 'Permutation importance for feature evaluation.',
+                            options: [
+                                { name: 'importance_featureData', label: 'Feature Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'X_train' },
+                                { name: 'importance_targetData', label: 'Target Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'y_train' },
+                                { name: 'scoring', component: ['option_suggest'], usePair: true, type: 'text', 
+                                    options: [
+                                        'explained_variance', 'max_error', 'neg_mean_absolute_error', 'neg_mean_squared_error', 'neg_root_mean_squared_error',
+                                        'neg_mean_squared_log_error', 'neg_median_absolute_error', 'r2', 'neg_mean_poisson_deviance', 'neg_mean_gamma_deviance',
+                                        'neg_mean_absolute_percentage_error'
+                                    ] },
+                                { name: 'sort', label: 'Sort data', component: ['bool_checkbox'], value: true, usePair: true },
+                                { name: 'importance_allocate', label: 'Allocate to', component: ['input'], placeholder: 'New variable', value: 'importances' }
+                            ]
+                        },
+                        'plot_permutation_importance': {
+                            name: 'plot_permutation_importance',
+                            label: 'Plot permutation importance',
+                            import: 'from sklearn.inspection import permutation_importance',
+                            code: 'vp_plot_permutation_importances(${model}, ${importance_featureData}, ${importance_targetData}${scoring}${sort}${top_count})',
+                            description: 'Permutation importance for feature evaluation.',
+                            options: [
+                                { name: 'importance_featureData', label: 'Feature Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'X_train' },
+                                { name: 'importance_targetData', label: 'Target Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'y_train' },
+                                { name: 'scoring', component: ['option_suggest'], usePair: true, type: 'text', 
+                                    options: [
+                                        'explained_variance', 'max_error', 'neg_mean_absolute_error', 'neg_mean_squared_error', 'neg_root_mean_squared_error',
+                                        'neg_mean_squared_log_error', 'neg_median_absolute_error', 'r2', 'neg_mean_poisson_deviance', 'neg_mean_gamma_deviance',
+                                        'neg_mean_absolute_percentage_error'
+                                    ] },
+                                { name: 'sort', label: 'Sort data', component: ['bool_checkbox'], value: true, usePair: true },
+                                { name: 'top_count', label: 'Top count', component: ['input_number'], min: 0, usePair: true }
+                            ]
+                        },
                         'Coefficient': {
                             name: 'coef_',
                             label: 'Coefficient',
@@ -631,7 +715,44 @@ define([
                                 { name: 'auc_targetData', label: 'Target Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'y_test' }
                             ]
                         },
-                        'permutation_importance': defaultInfos['permutation_importance']
+                        'permutation_importance': {
+                            name: 'permutation_importance',
+                            label: 'Permutation importance',
+                            import: 'from sklearn.inspection import permutation_importance',
+                            code: '${importance_allocate} = vp_create_permutation_importances(${model}, ${importance_featureData}, ${importance_targetData}${scoring}${sort})',
+                            description: 'Permutation importance for feature evaluation.',
+                            options: [
+                                { name: 'importance_featureData', label: 'Feature Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'X_train' },
+                                { name: 'importance_targetData', label: 'Target Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'y_train' },
+                                { name: 'scoring', component: ['option_suggest'], usePair: true, type: 'text', 
+                                    options: [
+                                        'accuracy', 'balanced_accuracy', 'top_k_accuracy', 'average_precision', 'neg_brier_score',
+                                        'f1', 'f1_micro', 'f1_macro', 'f1_weighted', 'f1_samples', 'neg_log_loss', 'precision', 'recall', 'jaccard', 
+                                        'roc_auc', 'roc_auc_ovr', 'roc_auc_ovo', 'roc_auc_ovr_weighted', 'roc_auc_ovo_weighted'
+                                    ] },
+                                { name: 'sort', label: 'Sort data', component: ['bool_checkbox'], value: true, usePair: true },
+                                { name: 'importance_allocate', label: 'Allocate to', component: ['input'], placeholder: 'New variable', value: 'importances' }
+                            ]
+                        },
+                        'plot_permutation_importance': {
+                            name: 'plot_permutation_importance',
+                            label: 'Plot permutation importance',
+                            import: 'from sklearn.inspection import permutation_importance',
+                            code: 'vp_plot_permutation_importances(${model}, ${importance_featureData}, ${importance_targetData}${scoring}${sort}${top_count})',
+                            description: 'Permutation importance for feature evaluation.',
+                            options: [
+                                { name: 'importance_featureData', label: 'Feature Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'X_train' },
+                                { name: 'importance_targetData', label: 'Target Data', component: ['data_select'], var_type: ['DataFrame', 'Series', 'ndarray', 'list', 'dict'], value: 'y_train' },
+                                { name: 'scoring', component: ['option_suggest'], usePair: true, type: 'text', 
+                                    options: [
+                                        'accuracy', 'balanced_accuracy', 'top_k_accuracy', 'average_precision', 'neg_brier_score',
+                                        'f1', 'f1_micro', 'f1_macro', 'f1_weighted', 'f1_samples', 'neg_log_loss', 'precision', 'recall', 'jaccard', 
+                                        'roc_auc', 'roc_auc_ovr', 'roc_auc_ovo', 'roc_auc_ovr_weighted', 'roc_auc_ovo_weighted'
+                                    ] },
+                                { name: 'sort', label: 'Sort data', component: ['bool_checkbox'], value: true, usePair: true },
+                                { name: 'top_count', label: 'Top count', component: ['input_number'], min: 0, usePair: true }
+                            ]
+                        },
                     }
 
                     // feature importances
@@ -779,6 +900,39 @@ define([
                         }
                     }
                     break;
+                case 'ETC':
+                    if (modelType === 'GridSearchCV') {
+                        infos = {
+                            'best_estimator_': {
+                                name: 'best_estimator_',
+                                label: 'Best estimator',
+                                code: '${best_estimator_allocate} = ${model}.best_estimator_',
+                                description: 'Estimator that was chosen by the search.',
+                                options: [
+                                    { name: 'best_estimator_allocate', label: 'Allocate to', component: ['input'], placeholder: 'New variable', value: 'best_estimator' }
+                                ]
+                            },
+                            'best_score_': {
+                                name: 'best_score_',
+                                label: 'Best score',
+                                code: '${best_score_allocate} = ${model}.best_score_',
+                                description: 'Mean cross-validated score of the best_estimator.',
+                                options: [
+                                    { name: 'best_score_allocate', label: 'Allocate to', component: ['input'], placeholder: 'New variable', value: 'best_score' }
+                                ]
+                            },
+                            'best_params_': {
+                                name: 'best_params_',
+                                label: 'Best params',
+                                code: '${best_params_allocate} = ${model}.best_params_',
+                                description: 'Parameter setting that gave the best results on the hold out data.',
+                                options: [
+                                    { name: 'best_params_allocate', label: 'Allocate to', component: ['input'], placeholder: 'New variable', value: 'best_params' }
+                                ]
+                            }
+                        }
+                    }
+                    break;
             }
             return infos;
         }
